@@ -1,17 +1,23 @@
 #! /usr/bin/python3
 # -*- coding: utf-8 -*-
 
-#####################################################################
-# THE SOFTWARE IS PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND,
-# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
-# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
-# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
-# OTHER DEALINGS IN THE SOFTWARE.
-#####################################################################
-"""Module container Checker Class.
+##############################################################################
+#    Copyright (C) 2021-current alexpdev
+#
+#    Licensed under the Apache License, Version 2.0 (the "License");
+#    you may not use this file except in compliance with the License.
+#    You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS,
+#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#    See the License for the specific language governing permissions and
+#    limitations under the License.
+##############################################################################
+"""
+Module container Checker Class.
 
 The CheckerClass takes a torrentfile and tha path to it's contents.
 It will then iterate through every file and directory contained
@@ -28,22 +34,27 @@ from pathlib import Path
 import pyben
 from tqdm import tqdm
 
-from .hasher import HasherHybrid, HasherV2
-from .utils import humanize_bytes
+from torrentfile.hasher import HasherHybrid, HasherV2
+from torrentfile.utils import MissingPathError
 
 SHA1 = 20
 SHA256 = 32
 
+logger = logging.getLogger(__name__)
+
 
 class Checker:
-    """Check a given file or directory to see if it matches a torrentfile.
+    """
+    Check a given file or directory to see if it matches a torrentfile.
 
     Public constructor for Checker class instance.
 
     Parameters
     ----------
-      metafile (`str`): Path to ".torrent" file.
-      location (`str`): Path where the content is located in filesystem.
+    metafile : str
+        Path to ".torrent" file.
+    path : str
+        Path where the content is located in filesystem.
 
     Example
     -------
@@ -56,14 +67,15 @@ class Checker:
 
     _hook = None
 
-    def __init__(self, metafile, path):
-        """Validate data against hashes contained in .torrent file.
+    def __init__(self, metafile: str, path: str):
+        """
+        Validate data against hashes contained in .torrent file.
 
         Parameters
         ----------
-        metafile : `str`
+        metafile : str
             path to .torrent file
-        path : `str`
+        path : str
             path to content or contents parent directory.
         """
         self.metafile = metafile
@@ -72,12 +84,15 @@ class Checker:
         self.paths = []
         self.fileinfo = {}
         self.last_log = None
+
         if not os.path.exists(metafile):
             raise FileNotFoundError
+
         self.meta = pyben.load(metafile)
         self.info = self.meta["info"]
         self.name = self.info["name"]
         self.piece_length = self.info["piece length"]
+
         if "meta version" in self.info:
             if "pieces" in self.info:
                 self.meta_version = 3
@@ -85,27 +100,30 @@ class Checker:
                 self.meta_version = 2
         else:
             self.meta_version = 1
+
         self.root = self.find_root(path)
         self.log_msg("Checking: %s, %s", metafile, path)
         self.check_paths()
 
     @classmethod
     def register_callback(cls, hook):
-        """Register hooks from 3rd party programs to access generated info.
+        """
+        Register hooks from 3rd party programs to access generated info.
 
         Parameters
         ----------
-        hook : `function`
+        hook : function
             callback function for the logging feature.
         """
         cls._hook = hook
 
     def hasher(self):
-        """Return the hasher class related to torrents meta version.
+        """
+        Return the hasher class related to torrents meta version.
 
         Returns
         -------
-        `Class[Hasher]`
+        hasher.Hasher
             the hashing implementation for specific torrent meta version.
         """
         if self.meta_version == 2:
@@ -115,42 +133,51 @@ class Checker:
         return None
 
     def piece_checker(self):
-        """Check individual pieces of the torrent.
+        """
+        Check individual pieces of the torrent.
 
         Returns
         -------
-        `Obj`
+        HashChecker | FeedChecker
             Individual piece hasher.
-        """ ""
+        """
         if self.meta_version == 1:
             return FeedChecker
         return HashChecker
 
     def results(self):
-        """Generate result percentage and store for future calls."""
+        """
+        Generate result percentage and store for future calls.
+        """
         if self.meta_version == 1:
             iterations = len(self.info["pieces"]) // SHA1
         else:
             iterations = (self.total // self.piece_length) + 1
+
         responses = []
         for response in tqdm(
             iterable=self.iter_hashes(),
             desc="Calculating",
+            leave=True,
             total=iterations,
-            unit="piece",
+            unit="pieces",
         ):
             responses.append(response)
-        print(responses)
+
+        self.log_msg(
+            "Final result for %s recheck:  %s", self.metafile, self._result
+        )
         return self._result
 
     def log_msg(self, *args, level=logging.INFO):
-        """Log message `msg` to logger and send `msg` to callback hook.
+        """
+        Log message `msg` to logger and send `msg` to callback hook.
 
         Parameters
         ----------
-        *args : `Iterable`[`str`]
+        args : dict
             formatting args for log message
-        level : `int`
+        level : int
             Log level for this message; default=`logging.INFO`
         """
         message = args[0]
@@ -162,12 +189,13 @@ class Checker:
         # Repeat log messages should be ignored.
         if message != self.last_log:
             self.last_log = message
-            logging.log(level, message)
+            logger.log(level, message)
             if self._hook and level == logging.INFO:
                 self._hook(message)
 
-    def find_root(self, path):
-        """Check path for torrent content.
+    def find_root(self, path: str) -> str:
+        """
+        Check path for torrent content.
 
         The path can be a relative or absolute filesystem path.  In the case
         where the content is a single file, the path may point directly to the
@@ -179,12 +207,13 @@ class Checker:
 
         Parameters
         ----------
-        path : `str`
+        path : str
             root path to torrent content
 
         Returns
         -------
-            `str`: root path to content
+        str
+            root path to content
         """
         if not os.path.exists(path):
             self.log_msg("Could not locate torrent content %s.", path)
@@ -202,47 +231,57 @@ class Checker:
         raise FileNotFoundError(root)
 
     def check_paths(self):
-        """Gather all file paths described in the torrent file."""
+        """
+        Gather all file paths described in the torrent file.
+        """
         finfo = self.fileinfo
+
         if "length" in self.info:
             self.log_msg("%s points to a single file", self.root)
             self.total = self.info["length"]
             self.paths.append(str(self.root))
+
             finfo[0] = {
                 "path": self.root,
                 "length": self.info["length"],
             }
+
             if self.meta_version > 1:
                 root = self.info["file tree"][self.name][""]["pieces root"]
                 finfo[0]["pieces root"] = root
+
             return
 
         # Otherwise Content is more than 1 file.
         self.log_msg("%s points to a directory", self.root)
         if self.meta_version == 1:
+
             for i, item in enumerate(self.info["files"]):
                 self.total += item["length"]
                 base = os.path.join(*item["path"])
+
                 self.fileinfo[i] = {
                     "path": str(self.root / base),
                     "length": item["length"],
                 }
+
                 self.paths.append(str(self.root / base))
-                self.log_msg("Including file path: %s", str(self.root / base))
             return
+
         self.walk_file_tree(self.info["file tree"], [])
 
-    def walk_file_tree(self, tree, partials):
-        """Traverse File Tree dictionary to get file details.
+    def walk_file_tree(self, tree: dict, partials: list):
+        """
+        Traverse File Tree dictionary to get file details.
 
         Extract full pathnames, length, root hash, and layer hashes
         for each file included in the .torrent's file tree.
 
         Parameters
         ----------
-        tree : `dict`
+        tree : dict
             File Tree dict extracted from torrent file.
-        partials : `list`
+        partials : list
             list of intermediate pathnames.
         """
         for key, val in tree.items():
@@ -251,8 +290,9 @@ class Checker:
             if "" in val:
 
                 base = os.path.join(*partials, key)
-                roothash = val[""]["pieces root"]
+                roothash = None
                 length = val[""]["length"]
+                roothash = None if not length else val[""]["pieces root"]
                 full = str(self.root / base)
                 self.fileinfo[len(self.paths)] = {
                     "path": full,
@@ -261,27 +301,22 @@ class Checker:
                 }
                 self.paths.append(full)
                 self.total += length
-                self.log_msg(
-                    "Including: path - %s, length - %s",
-                    full,
-                    humanize_bytes(length),
-                )
-
             else:
                 self.walk_file_tree(val, partials + [key])
 
-    def iter_hashes(self):
-        """Produce results of comparing torrent contents piece by piece.
+    def iter_hashes(self) -> tuple:
+        """
+        Produce results of comparing torrent contents piece by piece.
 
         Yields
         ------
-        chunck : `bytes`
+        chunck : bytes
             hash of data found on disk
-        piece : `bytes`
+        piece : bytes
             hash of data when complete and correct
-        path : `str`
+        path : str
             path to file being hashed
-        size : `int`
+        size : int
             length of bytes hashed for piece
         """
         matched = consumed = 0
@@ -289,42 +324,41 @@ class Checker:
         hasher = self.hasher()
         for chunk, piece, path, size in checker(self, hasher):
             consumed += size
-            msg = "Match %s: %s %s"
-            humansize = humanize_bytes(size)
             matching = 0
             if chunk == piece:
                 matching += size
                 matched += size
-                logging.debug(msg, "Success", path, humansize)
-            else:
-                logging.debug(msg, "Fail", path, humansize)
-            yield humansize, matching, consumed, matched
+            yield chunk, piece, path, size
             total_consumed = str(int(consumed / self.total * 100))
             percent_matched = str(int(matched / consumed * 100))
             self.log_msg(
                 "Processed: %s%%, Matched: %s%%",
                 total_consumed,
                 percent_matched,
+                level=logging.DEBUG,
             )
         self._result = (matched / consumed) * 100 if consumed > 0 else 0
 
 
 class FeedChecker:
-    """Validates torrent content.
+    """
+    Validates torrent content.
 
     Seemlesly validate torrent file contents by comparing hashes in
     metafile against data on disk.
 
     Parameters
     ----------
-    checker : `object`
+    checker : object
         the checker class instance.
-    hasher : `Any`
+    hasher : Any
         hashing class for calculating piece hashes. default=None
     """
 
-    def __init__(self, checker, hasher=None):
-        """Generate hashes of piece length data from filelist contents."""
+    def __init__(self, checker: Checker, hasher=None):
+        """
+        Generate hashes of piece length data from filelist contents.
+        """
         self.piece_length = checker.piece_length
         self.paths = checker.paths
         self.pieces = checker.info["pieces"]
@@ -336,32 +370,36 @@ class FeedChecker:
         self.it = None
 
     def __iter__(self):
-        """Assign iterator and return self."""
+        """
+        Assign iterator and return self.
+        """
         self.it = self.iter_pieces()
         return self
 
     def __next__(self):
-        """Yield back result of comparison."""
+        """
+        Yield back result of comparison.
+        """
         try:
             partial = next(self.it)
         except StopIteration as itererror:
             raise StopIteration from itererror
+
         chunck = sha1(partial).digest()  # nosec
         start = self.piece_count * SHA1
         end = start + SHA1
         piece = self.pieces[start:end]
         self.piece_count += 1
-        # except IndexError:  # pragma: no cover
-        #     raise StopIteration
         path = self.paths[self.index]
         return chunck, piece, path, len(partial)
 
     def iter_pieces(self):
-        """Iterate through, and hash pieces of torrent contents.
+        """
+        Iterate through, and hash pieces of torrent contents.
 
         Yields
         ------
-        piece : `bytes`
+        piece : bytes
             hash digest for block of torrent data.
         """
         partial = bytearray()
@@ -369,9 +407,9 @@ class FeedChecker:
             self.index = i
             if os.path.exists(path):
                 for piece in self.extract(path, partial):
-                    if len(piece) == self.piece_length:
-                        yield piece
-                    elif i + 1 == len(self.paths):
+                    if (len(piece) == self.piece_length) or (
+                        i + 1 == len(self.paths)
+                    ):
                         yield piece
                     else:
                         partial = piece
@@ -384,24 +422,27 @@ class FeedChecker:
                     else:
                         partial = pad
 
-    def extract(self, path, partial):
-        """Split file paths contents into blocks of data for hash pieces.
+    def extract(self, path: str, partial: bytearray) -> bytearray:
+        """
+        Split file paths contents into blocks of data for hash pieces.
 
         Parameters
         ----------
-        path : `str`
+        path : str
             path to content.
-        partial : `bytes`
+        partial : bytes
             any remaining content from last file.
 
-        Yields
-        ------
-        partial : `bytes`
+        Returns
+        -------
+        bytearray
             Hash digest for block of .torrent contents.
         """
         read = 0
         length = self.fileinfo[self.index]["length"]
         partial = bytearray() if len(partial) == self.piece_length else partial
+        if path not in self.paths:  # pragma: no cover
+            raise MissingPathError(path)
         with open(path, "rb") as current:
             while True:
                 bitlength = self.piece_length - len(partial)
@@ -419,21 +460,22 @@ class FeedChecker:
             for pad in self._gen_padding(partial, length, read):
                 yield pad
 
-    def _gen_padding(self, partial, length, read=0):
-        """Create padded pieces where file sizes do not match.
+    def _gen_padding(self, partial: bytes, length: int, read=0) -> bytes:
+        """
+        Create padded pieces where file sizes do not match.
 
         Parameters
         ----------
-        partial : `bytes`
+        partial : bytes
             any remaining data from last file processed.
-        length : `int`
+        length : int
             size of space that needs padding
-        read : `int`
+        read : int
             portion of length already padded
 
         Yields
         ------
-        `bytes`
+        bytes
             A piece length sized block of zeros.
         """
         while read < length:
@@ -451,18 +493,21 @@ class FeedChecker:
 
 
 class HashChecker:
-    """Verify that root hashes of content files match the .torrent files.
+    """
+    Verify that root hashes of content files match the .torrent files.
 
     Parameters
     ----------
-    checker : `Object`
+    checker : Object
         the checker instance that maintains variables.
-    hasher : `Object`
+    hasher : Object
         the version specific hashing class for torrent content.
     """
 
-    def __init__(self, checker, hasher=None):
-        """Construct a HybridChecker instance."""
+    def __init__(self, checker: Checker, hasher=None):
+        """
+        Construct a HybridChecker instance.
+        """
         self.checker = checker
         self.paths = checker.paths
         self.hasher = hasher
@@ -471,42 +516,41 @@ class HashChecker:
         self.piece_layers = checker.meta["piece layers"]
         self.piece_count = 0
         self.it = None
-        logging.debug(
-            "Starting Hash Checker. piece length: %s",
-            humanize_bytes(self.piece_length),
-        )
 
     def __iter__(self):
-        """Assign iterator and return self."""
+        """
+        Assign iterator and return self.
+        """
         self.it = self.iter_paths()
         return self
 
     def __next__(self):
-        """Provide the result of comparison."""
+        """
+        Provide the result of comparison.
+        """
         try:
             value = next(self.it)
             return value
         except StopIteration as stopiter:
             raise StopIteration() from stopiter
 
-    def iter_paths(self):
-        """Iterate through and compare root file hashes to .torrent file.
+    def iter_paths(self) -> tuple:
+        """
+        Iterate through and compare root file hashes to .torrent file.
 
         Yields
         ------
-        results : `tuple`
+        results : tuple
             The size of the file and result of match.
         """
         for i, path in enumerate(self.paths):
             info = self.fileinfo[i]
             length, plength = info["length"], self.piece_length
-            logging.debug("%s length: %s", path, str(length))
             roothash = info["pieces root"]
-            logging.debug("%s root hash %s", path, str(roothash))
             if roothash in self.piece_layers:
                 pieces = self.piece_layers[roothash]
             else:
-                pieces = roothash
+                pieces = roothash if roothash else bytes()
             amount = len(pieces) // SHA256
 
             if not os.path.exists(path):
@@ -520,13 +564,6 @@ class HashChecker:
                         size = length
                     length -= size
                     block = sha256(bytearray(size)).digest()
-                    logging.debug(
-                        "Yielding: %s %s %s %s",
-                        str(block),
-                        str(piece),
-                        path,
-                        str(size),
-                    )
                     yield block, piece, path, size
 
             else:
@@ -547,11 +584,4 @@ class HashChecker:
                             block = sha256(bytearray(size)).digest()
                         size = plength if plength < length else length
                         length -= size
-                        logging.debug(
-                            "Yielding: %s, %s, %s, %s",
-                            str(block),
-                            str(piece),
-                            str(path),
-                            str(size),
-                        )
                         yield block, piece, path, size
